@@ -8,20 +8,44 @@ fbGetMarketingStat <-
            filtering          = NULL,
            date_start         = Sys.Date() - 30,
            date_stop          = Sys.Date(),
+           attribution_window = NULL,
            api_version        = getOption("rfacebookstat.api_version"),
            action_report_time = NULL,
            interval           = "day",
            console_type       = "progressbar",
            request_speed      = "normal",
+           username           = getOption("rfacebookstat.username"),
+           token_path         = fbTokenPath(),
            access_token       = getOption("rfacebookstat.access_token")){
     
+   # auth 
+   if ( is.null(access_token) ) {    
+    
+     if ( Sys.getenv("RFB_API_TOKEN") != "" )  {
+	    access_token <- Sys.getenv("RFB_API_TOKEN")    
+ 	 } else {
+        access_token <- fbAuth(username   = username, 
+                               token_path = token_path)$access_token
+     }
+   }
+    
+    if ( class(access_token) == "fb_access_token" ) {
+      
+      access_token <- access_token$access_token
+      
+    }
+    
     # Check start time
     start_time <- Sys.time()
     
     # Create result DF
-    result <- data.table()
+    result <- tibble()
     
+    # attributes
+    rq_ids      <- list()
+    out_headers <- list()
     
+    # 
     # clear field list
     fields <- gsub("[\\s\\n\\t]", 
                    "", 
@@ -34,18 +58,17 @@ fbGetMarketingStat <-
                               action_breakdowns, 
                               perl = TRUE)
     
+    if ( ! is.null(attribution_window) ) {
+      
+      attribution_window <- tolower(attribution_window) %>% toJSON()
+      
+    }
+	
+	# check account format
+	accounts_id <- ifelse(grepl("^act_", accounts_id), accounts_id, paste0("act_",accounts_id))
+    
     # answer object class detect
-    answer_class <- ifelse ( action_breakdowns %in% c("action_device",
-                                                      "action_destination",
-                                                      "action_reaction",
-                                                      "action_target_id",
-                                                      "action_type",
-                                                      "action_type,action_reaction") &&
-                             ! is.null(action_breakdowns),
-                              action_breakdowns, 
-                              "actions")
-
-    if ( is.na(answer_class) ) answer_class <- "actions"
+    answer_class <- "actions"
     
     # filters handing to JSON
     if ( ! is.null(filtering) ) {
@@ -53,18 +76,18 @@ fbGetMarketingStat <-
         
         filters <- list()
         
-		for ( f in filtering ) {
-		  temp_fil    <- str_split(f, " ")
-		  temp_filter <- list(field    = temp_fil[[1]][1],
-							  operator = temp_fil[[1]][2])
-		  
-		  if ( temp_fil[[1]][2] %in% c("IN_RANGE", "NOT_IN_RANGE", "IN", "NOT_IN") ) {
-			temp_filter$value <- str_split(temp_fil[[1]][3], ",")[[1]]
-		  } else {
-			temp_filter$value <- temp_fil[[1]][3]
-		  }
-		  filters <- append(filters, list(temp_filter))
-		}
+        for ( f in filtering ) {
+          temp_fil    <- str_split(f, " ")
+          temp_filter <- list(field    = temp_fil[[1]][1],
+                              operator = temp_fil[[1]][2])
+          
+          if ( temp_fil[[1]][2] %in% c("IN_RANGE", "NOT_IN_RANGE", "IN", "NOT_IN") ) {
+            temp_filter$value <- str_split(temp_fil[[1]][3], ",")[[1]]
+          } else {
+            temp_filter$value <- temp_fil[[1]][3]
+          }
+          filters <- append(filters, list(temp_filter))
+        }
         filtering <- toJSON(filters, auto_unbox = T)
       }
     }
@@ -123,16 +146,22 @@ fbGetMarketingStat <-
         
         #Send API request
         answer <- httr::GET(QueryString,
-                            query = list(sort               = sorting,
-                                         level              = level,
-                                         breakdowns         = breakdowns,
-                                         action_breakdowns  = action_breakdowns,
-                                         fields             = fields,
-                                         filtering          = filtering,
-                                         action_report_time = action_report_time,
-                                         time_range         = str_interp("{\"since\":\"${dates_df$dates_from[dt]}\",\"until\":\"${dates_df$dates_to[dt]}\"}"),
-                                         limit              = 5000,
-                                         access_token       = access_token))
+                            query = list(sort                       = sorting,
+                                         level                      = level,
+                                         breakdowns                 = breakdowns,
+                                         action_breakdowns          = action_breakdowns,
+                                         fields                     = fields,
+                                         filtering                  = filtering,
+                                         action_report_time         = action_report_time,
+                                         action_attribution_windows = attribution_window,
+                                         time_range                 = str_interp("{\"since\":\"${dates_df$dates_from[dt]}\",\"until\":\"${dates_df$dates_to[dt]}\"}"),
+                                         limit                      = 5000,
+                                         access_token               = access_token))
+        
+        # attr
+        rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
+        out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
+        
         # check limit
         queryrep <- fbAPILimitCheck( answer, console_type, pb, pb_step, accounts_id, dates_df, pause_time  )
         
@@ -140,21 +169,27 @@ fbGetMarketingStat <-
         while ( queryrep ) {
           
           answer <- httr::GET(QueryString,
-                              query = list(sort               = sorting,
-                                           level              = level,
-                                           breakdowns         = breakdowns,
-                                           action_breakdowns  = action_breakdowns,
-                                           fields             = fields,
-                                           filtering          = filtering,
-                                           action_report_time = action_report_time,
-                                           time_range         = str_interp("{\"since\":\"${date_start}\",\"until\":\"${date_stop}\"}"),
-                                           limit              = 5000,
-                                           access_token       = access_token))
+                              query = list(sort                       = sorting,
+                                           level                      = level,
+                                           breakdowns                 = breakdowns,
+                                           action_breakdowns          = action_breakdowns,
+                                           fields                     = fields,
+                                           filtering                  = filtering,
+                                           action_report_time         = action_report_time,
+                                           action_attribution_windows = attribution_window,
+                                           time_range                 = str_interp("{\"since\":\"${dates_df$dates_from[dt]}\",\"until\":\"${dates_df$dates_to[dt]}\"}"),
+                                           limit                      = 5000,
+                                           access_token               = access_token))
+          
+          # attr
+          rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
+          out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
           
           queryrep <- fbAPILimitCheck( answer, console_type, pb, pb_step, accounts_id, dates_df, pause_time )
         }
         
         request_counter <- request_counter + 1
+        
         #Parse result
         answerobject <- httr::content(answer, as = "parsed")
         
@@ -189,10 +224,15 @@ fbGetMarketingStat <-
                                                fields             = fields,
                                                filtering          = filtering,
                                                action_report_time = action_report_time,
+                                               action_attribution_windows = attribution_window,
                                                time_range         = str_interp("{\"since\":\"${date_start}\",\"until\":\"${date_stop}\"}"),
                                                limit              = 5000,
                                                access_token       = access_token))
               
+            # attr
+            rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
+            out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
+              
               request_counter <- request_counter + 1
               answerobject <- httr::content(answer, as = "parsed")
               
@@ -223,15 +263,15 @@ fbGetMarketingStat <-
         }
         
         # action breakdown handing
-        if ( "actions" %in% unlist(str_split(fields, ",")) ) {
-
+        if ( any(c("actions", "action_values", "conversions", "conversion_values")  %in% unlist(str_split(fields, ","))) ) {
+          
           # switch functions
           class(answerobject) <- answer_class
           
           #Adding data to result
           tempData <- fbAction(answerobject)
           
-          result   <- rbind(result, tempData, fill = TRUE)
+          result   <- bind_rows(result, tempData)
           
           if (exists("tempData")) {
             rm(tempData)
@@ -240,7 +280,7 @@ fbGetMarketingStat <-
         } else {
           #Adding data to result
           tempData <- bind_rows(answerobject$data)
-          result   <- rbind(result, tempData, fill = TRUE)
+          result   <- bind_rows(result, tempData)
         }
         
         #Pagination
@@ -262,10 +302,15 @@ fbGetMarketingStat <-
                                              fields             = fields,
                                              filtering          = filtering,
                                              action_report_time = action_report_time,
+                                             action_attribution_windows = attribution_window,
                                              time_range         = str_interp("{\"since\":\"${date_start}\",\"until\":\"${date_stop}\"}"),
                                              limit              = 5000,
                                              access_token       = access_token))
             
+            # attr
+            rq_ids      <- append(rq_ids, setNames(list(status_code(answer)), answer$headers$`x-fb-trace-id`))
+            out_headers <- append(out_headers, setNames(list(headers(answer)), answer$headers$`x-fb-trace-id`))
+            
             queryrep <- fbAPILimitCheck( answer, console_type, pb, pb_step, accounts_id, dates_df, pause_time )
           }
           
@@ -321,7 +366,7 @@ fbGetMarketingStat <-
             #Adding data to result
             tempData <- fbAction(answerobject)
             
-            result   <- rbind(result, tempData, fill = TRUE)
+            result   <- bind_rows(result, tempData)
             
             if (exists("tempData")) {
               rm(tempData)
@@ -330,12 +375,36 @@ fbGetMarketingStat <-
           } else {
             #Adding data to result
             tempData <- bind_rows(answerobject$data)
-            result   <- rbind(result, tempData, fill = TRUE)
+            result   <- bind_rows(result, tempData,)
           }
+        }
       }
-     }
+    }
+    
+    # prepare output
+    # unnesting_values
+    vlist <- select_if(result, is.list) %>% names()
+
+    for ( l in vlist ) {
+
+      result <- unnest_wider(result, l, names_sep = "_") %>%
+                      select(-matches("^.*\\_action\\_type"))
+
     }
 
+    # renaming
+    newnames <- names(result) %>%
+                str_remove_all("NA") %>%
+                str_remove("^\\_") %>%
+                str_replace("\\_\\_", "\\_") %>%
+                str_remove("\\_value$")
+   
+    names(result) <- newnames
+    
+    # set attributes
+    attr(result, "request_ids") <- rq_ids
+    attr(result, "headers")     <- out_headers
+    
     if(console_type == "progressbar"){  
       #Progressbar close
       utils::setTxtProgressBar(pb, length(accounts_id) * nrow(dates_df))
